"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/param/__init__.py $
$Id: __init__.py 28119 2006-03-31 21:57:52Z rmasse $
"""
from dulcinea.common import get_publisher

def get_parameter_db():
    return get_publisher().get_connection().get_root().get('parameter_db')
