"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/dulcinea/lib/__init__.py $
$Id: __init__.py 28638 2006-08-22 13:20:02Z dbinger $
"""
__version__ = "0.15"

try:
    import dulcinea.common
except ImportError:
    print "Dulcinea", __version__
